# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

from __future__ import unicode_literals, print_function, division
from builtins import range
from builtins import str as unicode

from pprint import pprint

import sys
import getpass

import TestData_Py3DGeom as test
from Py3DGeom import *
import Py3DGeom_py 

import time

con_tuple = ('scott','tiger','localhost:1521/orcl')

create = "CREATE TABLE TABLE2 (ID VARCHAR(20), GEOM MDSYS.SDO_GEOMETRY)"

statement = "INSERT INTO TABLE2 (ID, GEOM) VALUES (%s, %s)"

query = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM TABLE2 a WHERE ID = %s"

query_1 = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM TABLE1 a WHERE ID = %s" 


def coord_frame(lst):

    return pd.DataFrame(np.array(lst).reshape(int(len(lst)/3.0), 3))


def full_test(verbose = True):
    
    con = cx.connect(*con_tuple)    
    cur = con.cursor()
    cur.execute("DELETE FROM TABLE2")
    
    p = []
    print("starting tests")
    t1 = time.time()
    p.append(test_point(cur, verbose))
    t2 = time.time()
    p.append(test_line(cur,verbose))
    t3 = time.time()
    p.append(test_simple_polygon(cur,verbose))
    t4 = time.time()
    p.append(test_polygon(cur,verbose))    
    t5 = time.time()
    p.append(test_surface(cur,verbose))  
    t6 = time.time()
    p.append(test_solid(cur,verbose))
    t7 = time.time()

    

    #p.append(test_occ_plot(cur, verbose = False))

    
    if all(p):
        print("All tests passed")
        
    cur.close()
    con.commit()
    con.close()

def test_point(cur, verbose = False):
    
    p = []
    a = SDO_Geometry(Point())
    p.append(str(a.SDO_GEOMETRY) == test.point_1)
    
    a = SDO_Geometry(Point(1,1,1))
    p.append(str(a.SDO_GEOMETRY) == test.point_2)
    
    
    b=Point()
    b.set_point(eu.Point3(2.2,2.3,2.4))
    a=SDO_Geometry(b)
    p.append(str(a.SDO_GEOMETRY) == "SDO_GEOMETRY (3001, 7405, SDO_POINT_TYPE (2.2, 2.3, 2.4), NULL, NULL)")    
    
    a = SDO_Geometry(None, 3001, 7405, None, [1,1,1], [3.1,3.2,3.3])
    p.append(str(a.SDO_GEOMETRY) == "SDO_GEOMETRY (3001, 7405, SDO_POINT_TYPE (3.1, 3.2, 3.3), NULL, NULL)")    
    
    cur.execute(statement%(1, a.SDO_GEOMETRY))
    
    cur.execute(statement%(3,a.SDO_GEOMETRY))
    a.setAltPt(False)  

    cur.execute(query%3)
    element = cur.fetchone()
    
    if element[2] != None:
        a = SDO_Geometry(None, element[0],element[1],(element[2].X,element[2].Y,element[2].Z),None,None)
    elif element[3]:
        a = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())    
    
    
    p.append(str(a.SDO_GEOMETRY) == "SDO_GEOMETRY (3001, 7405, SDO_POINT_TYPE (3.1, 3.2, 3.3000000000000003), NULL, NULL)")
    
    if all(p):
        if verbose:
            print("Passed point test")
        return True
    else:
        if verbose:
            print("Failed point test :%s"%p)
        return False

def test_line(cur, verbose = False):

    p = []
    
    a = LineString()
    b = SDO_Geometry(a)  
    p.append(repr(b)== "SDO_GEOMETRY 3002")


    arr = np.array([[1,2,3,4,5,6,7,8,9]], float).reshape(int(9/3.0),3)
    a = LineString(arr)
    b = SDO_Geometry(a) 
    
    p.append(b.SDO_GEOMETRY == test.line_1)

    cur.execute(statement%(4,b.SDO_GEOMETRY))
    
    cur.execute(query%4)

    element = cur.fetchone()

    if element[2] != None:
        c = SDO_Geometry(None, element[0],element[1],(element[2].X,element[2].Y,element[2].Z),None,None)
    elif element[3]:
        c = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())

    p.append(c.SDO_GEOMETRY ==test.line_1)

    if all(p):
        if verbose:
            print("Passed linestring test")
        return True
    else:
        if verbose:
            print("Failed linestring test :%s"%p)
        return False    

def test_simple_polygon(cur, verbose = False):

    p = []

    arr = np.array([[0,0,0,0,1,0,1,1,0,0,0,0]], float).reshape(int(12/3.0),3)
        
    a = Polygon(PolygonRing(arr), simple = True)
    b = SDO_Geometry(a)
    
    p.append(b.SDO_GEOMETRY =='SDO_GEOMETRY (3003, 7405, NULL, SDO_ELEM_INFO_ARRAY (1, 3, 1), SDO_ORDINATE_ARRAY (0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0))')
    
        
    cur.execute(statement%(5,str(b.SDO_GEOMETRY)))

    cur.execute(query%5)
    element = cur.fetchone()
    
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())
    p.append(b.SDO_GEOMETRY =='SDO_GEOMETRY (3003, 7405, NULL, SDO_ELEM_INFO_ARRAY (1, 3, 1), SDO_ORDINATE_ARRAY (0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0))')


    if all(p):
        if verbose:
            print("Passed simple polygon test")
        return True
    else:
        if verbose:
            print("Failed simple polygon test :%s"%p)
        return False    

def test_polygon(cur, verbose = False):

    p = []
    
    corners = SDO_Geometry(None, 3003, 7405, None, [1,3,3], [0.0,0.0,0.0,1.0,1.0,0.0])
    corners = SDO_Geometry(None, 3003, 7405, None, [1,3,3], [0.0,0.0,0.0,0.0,1.0,1.0])
    corners = SDO_Geometry(None, 3003, 7405, None, [1,3,3], [0.0,0.0,0.0,1.0,0.0,1.0])   

    b = SDO_Geometry(None, 3003, 7405, None, [1,1003,1], [0,0,0,0,1,0,1,1,0,0,0,0] )

    p.append(b.SDO_GEOMETRY ==test.poly_1)    

    cur.execute(statement%(6,str(b.SDO_GEOMETRY)))
    cur.execute(query%6) 
    element = cur.fetchone()   
    
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())
    p.append(b.SDO_GEOMETRY =='SDO_GEOMETRY (3003, 7405, NULL, SDO_ELEM_INFO_ARRAY (1, 1003, 1), SDO_ORDINATE_ARRAY (0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0))')
    
    
    a = Polygon(PolygonRing( np.array([0.0,0.0,0.0,0.0,1.0,0.0,1.0,1.0,0.0,0.0,0.0,0.0]).reshape(4,3) ))
    hole = PolygonRing( np.array([0.1,0.1,0.0,0.1,0.9,0.0,0.9,0.9,0,0.1,0.1,0]).reshape(4,3), interior = True)
    a.addHole([hole])
    b=SDO_Geometry(a)
    
    cur.execute(statement%(7,str(b.SDO_GEOMETRY)))
    
    cur.execute(query%7) 
    element = cur.fetchone()
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())     
    p.append((b.SDO_GEOMETRY == test.poly_2 )| (b.SDO_GEOMETRY == test.poly_2A))

    if all(p):
        if verbose:
            print("Passed polygon test")
        return True
    else:
        if verbose:
            print("Failed polygon test :%s"%p)
        return False    
    

def test_surface(cur, verbose = False):

    p = []
    
    polyhole = PolygonRing(np.array([4.0, 4.0, 0.0,
                                     4.0, 6.0, 0.0,
                                     6.0, 6.0, 0.0,
                                     6.0, 4.0, 0.0,
                                     4.0, 4.0, 0.0]).reshape(5,3))
    
    
    poly = Polygon(PolygonRing(np.array([0.0, 0.0, 0.0,
                                         0.0, 10.0, 0.0,
                                         10.0, 10.0, 0.0,
                                         10.0, 0.0, 0.0,
                                         0.0, 0.0, 0.0 ]).reshape(5,3)), 
                   holes = [polyhole])


    secondpoly = Polygon(PolygonRing(np.array([10.0, 0.0, 0.0,
                                               20.0, 0.0, 0.0,
                                               20.0, 10.0, 0.0,
                                               10.0, 10.0, 0.0,
                                               10.0, 0.0, 0.0])))


    surf = Surface([poly])
    surf.addPolygon([secondpoly])
    a = SDO_Geometry(surf)
    #print (pd.DataFrame(np.array(surf.ord_info).reshape(int(len(surf.ord_info)/3.0), 3)))
    

    p.append(a.SDO_GEOMETRY == test.surf_1)

    cur.execute(statement%(8,str(a.SDO_GEOMETRY)))
    cur.execute(query%8) 
    element = cur.fetchone()
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())          

    p.append(b.SDO_GEOMETRY == test.surf_1)

    if all(p):
        if verbose:
            print("Passed surface test")
        return True
    else:
        if verbose:
            print("Failed surface test :%s"%p)
        return False        

def test_solid(cur, verbose = False):

    global a, b
    
    p = []
    
    cur.execute(query%8) 
    element = cur.fetchone()
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())     
    
    simsol = SimpleSolid(b.geom)
    a = SDO_Geometry(simsol)
    p.append(a.SDO_GEOMETRY==test.solid_1)
    
    
    simsol2 = SimpleSolid(b.geom, int_surf = [b.geom])
    a2 = SDO_Geometry(simsol2)
    p.append(a2.SDO_GEOMETRY==test.solid_2)
    

    cur.execute(query_1%40)
    element = cur.fetchone()
    t1 = time.time()
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())  
    t2 = time.time()
    b1 = Py3DGeom_py.SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())  
    t3 = time.time()
    
    #print ("Time %s,%s%%"%(t2-t1, (t2-t1)*100/(t3-t2)))
    
    p.append(b.SDO_GEOMETRY==test.solid_3)
    
    if all(p):
        if verbose:
            print("Passed solid test")
        return True
    else:
        if verbose:
            print("Failed solid test :%s"%p)
        return False
    
def test_plot(cur, verbose = False):
    
    p = [True]
    
    plot = Plot_mlab()

    query_1 = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
    a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM TABLE1 a WHERE ID = %s"     
    con = cx.connect('scott','tiger','localhost:1521/orcl')    
    cur = con.cursor()
    cur.execute(query_1%40)
    element = cur.fetchone()

    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())

    b.geom.plot(plot)

    plot.setLimits()

    plt.show() 
    
    if all(p):
        if verbose:
            print("Passed plot test")
        return True
    else:
        if verbose:
            print("Failed plot test :%s"%p)
        return False    
    

def test_real_data(cur, verbose = False):
    
    p =[]

    #query_1 = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
    #a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM A20170528_ELEMENTS a WHERE GUID = '%s'"     
    #print   (query_1%"2RqNejBvj5BB_yuM07PAaF")
    
    query_1 = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
    a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM A20170528_ELEMENTS a "     
    
    cur.execute(query_1)

    fig, ax, cc, min_b, max_b = init_plot()
    
    keepgoing = True 
    while keepgoing:        
        element = cur.fetchone()
        if element:       
            b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist()) 
       
            b.geom.plot(ax, cc, min_b, max_b)
        else:
            keepgoing = False
            
    
    
    print (max_b[0] , min_b[0], max_b[0] - min_b[0])
    
    
    
    setLimits(ax, min_b, max_b) 
    
    
    plt.show()

    if all(p):
        if verbose:
            print("Passed real data test")
        return True
    else:
        if verbose:
            print("Failed real data test :%s"%p)
        return False
    
    
def test_occ_plot(cur, verbose = False):
    
    from pythonocc.Display import SimpleGui
    
    import OCC.Core.AIS as ais
    import OCC.Core.BRepBuilderAPI as brep
    import OCC.Core.GC as gc
    import OCC.Core.gp as gp
    import OCC.Core.TopoDS as tp    
    
    p = [True]
    
    query_1 = "SELECT a.GEOM.SDO_GTYPE, a.GEOM.SDO_SRID, a.GEOM.SDO_POINT,\
    a.GEOM.SDO_ELEM_INFO, a.GEOM.SDO_ORDINATES FROM TABLE1 a WHERE ID = %s" 
    
    con = cx.connect('scott','tiger','localhost:1521/orcl')    
    cur = con.cursor()
    cur.execute(query_1%40)
    element = cur.fetchone()
    
    b = SDO_Geometry(None, element[0],element[1],None,element[3].aslist(),element[4].aslist())
    
    disp = {}
    display = SimpleGui.init_display("qt-pyqt5", disp = disp, perspective=True)
    b.geom.set_colour()
    b.geom.plotocc(display)
    display.FitAll()
    disp['start']()    
    
    if all(p):
        if verbose:
            print("Passed OCC plot test")
        return True
    else:
        if verbose:
            print("Failed OCC plot :%s"%p)
        return False        
    



if (__name__ == '__main__'):

    arg_defaults = []    # Argument names for passing to main()

    arg_names = []
    arg_values = []

    while arg_defaults:
        arg_names.append(arg_defaults.pop(0))
        arg_values.append(arg_defaults.pop(0))

    kwargs = {}             # Dictionary for holding main() arguments

    args = sys.argv[1:len(arg_names)+1] # Arguments not including module 

    debug = False
    proceed = True

    if len(args) != 0:      # Populate kwargs with arguments

        for n,each in enumerate(arg_values):
            kwargs[arg_names[n]] = each 

        for n,each in enumerate(sys.argv[1:len(args)+1]):
            #if debug: print("%s, %s"%(n,each))
            kwargs[arg_names[n]] = each


    else:  # Default arguments     

        for n,each in enumerate(arg_values):
            kwargs[arg_names[n]] = each

    if debug:        
        pprint(kwargs)


    if proceed:
        full_test(**kwargs) 