#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# cython3 compatible

# Copyright Gareth Boyes, 2021

import sys
import numpy as np

from OCCUtils.Topology import shapeTypeString, WireExplorer

from OCC.Core.BRep import BRep_Tool
from OCC.Core.TopAbs import TopAbs_VERTEX, TopAbs_WIRE, TopAbs_SOLID, TopAbs_FACE, TopAbs_SHELL, TopAbs_COMPOUND
from OCC.Core.TopoDS import topods_Vertex, TopoDS_Iterator, topods_Wire
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakeVertex
from OCC.Core.gp import gp_Pnt


import Py3DGeom

def convertTopology(myshape, level=0, closed=False):
    ret = None
    brt = BRep_Tool()
    s = myshape.ShapeType()
    
    if s == TopAbs_VERTEX:
        pnt = brt.Pnt(topods_Vertex(myshape))
        #print(".." * level  + "<Vertex %i: %s %s %s>" % (hash(myshape), pnt.X(), pnt.Y(), pnt.Z()))
    
    elif s == TopAbs_WIRE:
        wire = WireExplorer(topods_Wire(myshape))
        #print(".." * level + "<Wire %i>" %(hash(myshape)))
        coords = []
        
        for n,each in enumerate(wire.ordered_vertices()):
            if n==0:
                first = each
            #convertTopology(each, level + 2)

            coords.extend(brt.Pnt(each).Coord())
        if len(coords) == 0:
            return None
        
        if closed:
            coords.extend(brt.Pnt(first).Coord())
        c = np.array(coords)
        c = c.reshape((int(c.shape[0]/3),3))
        
        wire = Py3DGeom.PolygonRing(c)
        return wire
            
    elif s == TopAbs_SOLID:     
        it = TopoDS_Iterator(myshape)
        shells = []
        while it.More():
            shp = it.Value()
            it.Next()
            shells.append(convertTopology(shp, level + 1))        
        
        
        solid = Py3DGeom.SimpleSolid(shells[0])
        return solid
        
    elif s == TopAbs_SHELL:
        it = TopoDS_Iterator(myshape)
        faces = []
        while it.More():
            shp = it.Value()
            
            this_face = convertTopology(shp, level + 1)
            if this_face is not None:
                faces.append(this_face)        
            it.Next()
        
        shell = Py3DGeom.Surface(faces)
        return shell
        
    elif s == TopAbs_FACE:
       
        it = TopoDS_Iterator(myshape)
        wires = []
        while it.More():
            shp = it.Value()
            it.Next()
            wires.append(convertTopology(shp, level + 1, closed = True))     
        if wires[0] is None:
            face = None
        else:
            face = Py3DGeom.Polygon(wires[0])        
        
        return face
    elif s == TopAbs_COMPOUND:
        it = TopoDS_Iterator(myshape)
        type_list = []
        while it.More():
            shp = it.Value()
            it.Next()
            it_shape = shp.ShapeType()
            type_list.append(it_shape)
            
        if all([each==TopAbs_FACE for each in type_list]):
            faces = []
            it = TopoDS_Iterator(myshape)
            while it.More():
                shp = it.Value()
                this_face = convertTopology(shp, level + 1)
                if this_face is not None:
                    faces.append(this_face)        
                it.Next()          
            surface = Py3DGeom.Surface(faces)
            return surface
        
        elif all([each==TopAbs_SHELL for each in type_list]): 
            shells = []
            it = TopoDS_Iterator(myshape)
            while it.More():
                shp = it.Value()
                it.Next()
                shells.append(shp)   
            faces = []

            for each_shell in shells:

                it = TopoDS_Iterator(each_shell)
                while it.More():
                    shp = it.Value()
                    this_face = convertTopology(shp, level + 1)
                    if this_face is not None:
                        faces.append(this_face)        
                    it.Next()
                
            shell = Py3DGeom.Surface(faces)
            return shell                
                
        else:
            raise Exception("Not Surface")
                
    else:
        it = TopoDS_Iterator(myshape)
        while it.More():
            shp = it.Value()
            it.Next()
            convertTopology(shp, level + 1)