#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

from itertools import product

import numpy as np
import cython as cy

# Logging
from utils.logging_env import setup_logging, logging
log = logging.getLogger(__name__)
setup_logging(log)

SQRT2 = 1.4142135623730951
SQRT3 = 1.7320508075688772

adjacency_7 : tuple =   ((1,0,0,False,1),
                        (0,1,0,False,1),
                        (0,0,1,False,1),
                        
                        (1,1,0,True,SQRT2),
                        (0,1,1,True,SQRT2),
                        (1,0,1,True,SQRT2),
                        
                        (1,1,1,True,SQRT3))
                 

def iterate(array):
    
    sx: cy.int = array.shape[0]
    sy: cy.int = array.shape[1]
    sz: cy.int = array.shape[2]    
    
    x: cy.int 
    y: cy.int
    z: cy.int 
        
    dx: cy.int 
    dy: cy.int
    dz: cy.int    
    
    dist: cy.float
    diag : cy.bint
    
    diag_manifold_conflict: cy.bint = False
    edge_manifold_conflict: cy.bint = False
    
    debug = False
    
    for x, y, z in product(range(sx), range(sy), range(sz)):
        
        

        if x == sx - 1 or y == sy - 1 or z == sz -1:
            pass
        else:

                
            if array[x:x+2, y:y+2, z:z+2].sum() == 2:   
                if array[x,y,z]  and array [x+1,y+1,z+1] :        
                    diag_manifold_conflict = True
                    
                elif array [x+1, y, z]  and array [x,y+1,z+1] :  
                    diag_manifold_conflict = True
                    
                elif array [x, y+1, z] and array [x+1,y,z+1] :  
                    diag_manifold_conflict = True
                    
                elif array [x+1, y+1, z]  and array [x,y,z+1]:  
                    diag_manifold_conflict = True
            
            elif array[x:x+2, y:y+2, z:z+2].sum() == 6:   
                if not array[x,y,z]  and not array [x+1,y+1,z+1] :        
                    diag_manifold_conflict = True
                    
                elif not array [x+1, y, z]  and not array [x,y+1,z+1] :  
                    diag_manifold_conflict = True
                    
                elif not array [x, y+1, z] and not array [x+1,y,z+1] :  
                    diag_manifold_conflict = True
                    
                elif not array [x+1, y+1, z]  and not array [x,y,z+1]:  
                    diag_manifold_conflict = True             
        
        x_cut = array[x,:,:]    
        y_cut = array[:,y,:]
        z_cut = array[:,:,z]
        
        #if x==0 and y==0 and z==0:
            #print (x_cut)
            #print (x_cut[y:y+2, z:z+2])
            
            #print (y_cut)
            #print (y_cut[x:x+2, z:z+2])
            
            #print (z_cut)
            #print (z_cut[x:x+2, y:y+2])
        
        if y == sy - 1 or z == sz -1:
            pass
        else:
        
            if x_cut[y:y+2, z:z+2].sum() == 2:
                if x_cut[y,z] == 1 and x_cut[y+1,z+1] ==1:        
                    edge_manifold_conflict = True
                    
                elif x_cut[y+1,z] == 1 and x_cut[y,z+1] ==1:     
                    edge_manifold_conflict = True 
                    
        if x == sx - 1 or z == sz -1:
            pass
        else:
            if y_cut[x:x+2, z:z+2].sum() == 2:
                if y_cut[x,z] == 1 and y_cut[x+1,z+1] ==1:        
                    edge_manifold_conflict = True
                    
                elif y_cut[x,z+1] == 1 and y_cut[x+1, z] ==1:     
                    edge_manifold_conflict = True 
                    
        if x == sx - 1 or y == sy - 1:
            pass
        else:
            if z_cut[x:x+2, y:y+2].sum() == 2:
                if z_cut[x,y] == 1 and z_cut[x+1,y+1] ==1:        
                    edge_manifold_conflict = True
                elif z_cut[x+1,y] == 1 and z_cut[x,y+1] ==1:     
                    edge_manifold_conflict = True

                    
        if edge_manifold_conflict and diag_manifold_conflict:
            break
        
    return diag_manifold_conflict, edge_manifold_conflict
            
def iterate2(array, array2):
    
    # print("two arrays")
    
    sx: cy.int = array.shape[0]
    sy: cy.int = array.shape[1]
    sz: cy.int = array.shape[2]    
    
    x: cy.int 
    y: cy.int
    z: cy.int 
        
    dx: cy.int 
    dy: cy.int
    dz: cy.int    
    
    dist: cy.float
    diag : cy.bint
    

    external_diag_manifold_conflict: cy.bint = False
    external_edge_manifold_conflict: cy.bint = False
    
    internal_diag_manifold_conflict: cy.bint = False
    internal_edge_manifold_conflict: cy.bint = False    
    
    for x, y, z in product(range(sx), range(sy), range(sz)):
        
        potential_diag_manifold_conflict: cy.bint = False
        potential_edge_manifold_conflict: cy.bint = False        
        
        if x==5 and y==2 and z==2:
            
            debug = True
        else:
            debug = False        
        
        
        if x == sx - 1 or y == sy - 1 or z == sz -1:
            pass
        else:
            if array[x:x+2, y:y+2, z:z+2].sum() == 2:   
                if array[x,y,z] == 1 and array [x+1,y+1,z+1] ==1:        
                    potential_diag_manifold_conflict = True
                    
                elif array [x+1, y, z] == 1 and array [x,y+1,z+1] ==1:  
                    potential_diag_manifold_conflict = True
                    
                elif array [x, y+1, z] == 1 and array [x+1,y,z+1] ==1:  
                    potential_diag_manifold_conflict = True
                    
                elif array [x+1, y+1, z] == 1 and array [x,y,z+1] ==1:  
                    potential_diag_manifold_conflict = True
             
            elif array[x:x+2, y:y+2, z:z+2].sum() == 6:   
                if not array[x,y,z] and not array [x+1,y+1,z+1]:        
                    potential_diag_manifold_conflict = True
                    
                elif not array [x+1, y, z] and not array [x,y+1,z+1]:  
                    potential_diag_manifold_conflict = True
                    
                elif not array [x, y+1, z] and not array [x+1,y,z+1]:  
                    potential_diag_manifold_conflict = True
                    
                elif not array [x+1, y+1, z] and not array [x,y,z+1]:  
                    potential_diag_manifold_conflict = True                  
            
        
        if potential_diag_manifold_conflict == True:
            
            if array2[x:x+2, y:y+2, z:z+2].sum() == 2:   
                if array2[x,y,z] and array2 [x+1,y+1,z+1]:        
                    internal_diag_manifold_conflict = True
                
                elif array2[x+1, y, z] and array2[x,y+1,z+1]:  
                    internal_diag_manifold_conflict = True           
                    
                elif array2[x, y+1, z] and array2[x+1,y,z+1]:  
                    internal_diag_manifold_conflict = True              
                    
                elif array2[x+1, y+1, z] and array2[x,y,z+1]:  
                    internal_diag_manifold_conflict = True            
                else:
                    external_diag_manifold_conflict = True                
            
            elif array2[x:x+2, y:y+2, z:z+2].sum() == 6:   
                if not array2[x,y,z]  and not array2 [x+1,y+1,z+1]:        
                    internal_diag_manifold_conflict = True
                
                elif not array2[x+1, y, z] and not array2[x,y+1,z+1]:  
                    internal_diag_manifold_conflict = True           
                    
                elif not array2[x, y+1, z] and not array2[x+1,y,z+1]:  
                    internal_diag_manifold_conflict = True              
                    
                elif not array2[x+1, y+1, z] and not array2[x,y,z+1] ==1:  
                    internal_diag_manifold_conflict = True            
                else:
                    external_diag_manifold_conflict = True  
            else:
                external_diag_manifold_conflict = True              
        
        x_cut = array[x,:,:]    
        y_cut = array[:,y,:]
        z_cut = array[:,:,z]
        
        x_cut2 = array2[x,:,:]    
        y_cut2 = array2[:,y,:]
        z_cut2 = array2[:,:,z]        
        
        #if x==0 and y==0 and z==0:
            #print (x_cut)
            #print (x_cut[y:y+2, z:z+2])
            
            #print (y_cut)
            #print (y_cut[x:x+2, z:z+2])
            
            #print (z_cut)
            #print (z_cut[x:x+2, y:y+2])
        
        potential_edge_manifold_conflict = False
        
        if y == sy - 1 or z == sz -1:
            pass
        else:

            if x_cut[y:y+2, z:z+2].sum() == 2:
                # if debug:
                    
                    # print(x_cut[y,z] and x_cut[y+1,z+1])
                    # print(x_cut[y+1,z] and x_cut[y,z+1])
                if x_cut[y,z] and x_cut[y+1,z+1]:        
                    potential_edge_manifold_conflict = True
                    
                elif x_cut[y+1,z] and x_cut[y,z+1]:     
                    potential_edge_manifold_conflict = True  
                    
                if potential_edge_manifold_conflict == True:
                    # print(x_cut2[y:y+2, z:z+2].sum())
                    if x_cut2[y:y+2, z:z+2].sum() == 2:
                        if x_cut2[y,z] and x_cut2[y+1,z+1]:        
                            internal_edge_manifold_conflict = True
                            
                        elif x_cut2[y+1,z] and x_cut2[y,z+1]:     
                            internal_edge_manifold_conflict = True 
                        else:
                            external_edge_manifold_conflict = True
                    else:
                        external_edge_manifold_conflict = True
                        
        potential_edge_manifold_conflict = False
        
        if x == sx - 1 or z == sz -1:
            pass
        else:
       
            if y_cut[x:x+2, z:z+2].sum() == 2:
                if y_cut[x,z] == 1 and y_cut[x+1, z+1] ==1:        
                    potential_edge_manifold_conflict = True
                    
                elif y_cut[x,z] == 1 and y_cut[x, z+1] ==1:     
                    potential_edge_manifold_conflict = True 
            
                if potential_edge_manifold_conflict == True:
                
                    if y_cut2[x:x+2,z:z+2].sum() == 2:
                        if y_cut2[x,z] == 1 and y_cut2[x+1, z+1] ==1:        
                            internal_edge_manifold_conflict = True
                            
                        elif y_cut2[x+1, z] == 1 and y_cut2[x,z] ==1:     
                            internal_edge_manifold_conflict = True
                            
                        else:
                            external_edge_manifold_conflict = True
                    else:
                        external_edge_manifold_conflict = True                    

        potential_edge_manifold_conflict = False

        if x == sx - 1 or y == sy - 1:
            pass
        else:
            
            # if debug:
                # print(z_cut)
                # print(z_cut2)
                # print(x,x+2, y, y+2)
                # print(z_cut[x:(x+2), y:(y+2)])
                # print(z_cut[x:x+2, y:y+2].sum())                 
            
            if z_cut[x:x+2, y:y+2].sum() == 2:
                
                if z_cut[x,y] == 1 and z_cut[x+1,y+1] ==1:        
                    potential_edge_manifold_conflict = True
                elif z_cut[x+1,y] == 1 and z_cut[x,y+1] ==1:     
                    potential_edge_manifold_conflict = True

                if potential_edge_manifold_conflict == True:

                    if z_cut2[x:x+2, y:y+2].sum() == 2:
                        if z_cut2[x,y] == 1 and z_cut2[x+1,y+1] ==1:        
                            internal_edge_manifold_conflict = True
                        elif z_cut2[x+1,y] == 1 and z_cut2[x,y+1] ==1:     
                            internal_edge_manifold_conflict = True                

                        else:
                            external_edge_manifold_conflict = True
                    else:
                        external_edge_manifold_conflict = True                    

                    
        # if edge_manifold_conflict and diag_manifold_conflict:
            # break
        
    return (internal_diag_manifold_conflict, internal_edge_manifold_conflict,
            external_diag_manifold_conflict, external_edge_manifold_conflict)
            
