# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

import json

from settings.load_settings import get_progress_settings, get_display_settings


class Progress(object):

    def __init__(self, progress_no=None, display_no=None, load=None,
                 update={}, display_update={}):

        if progress_no is None:
            progress_no = "_default"
        if display_no is None:
            display_no = "_default"

        self.progress_no = progress_no
        self.display_no = display_no

        self.s, s = get_progress_settings(progress_no)
        self.s = self.s._replace(**update)
        self.d, s = get_display_settings(display_no)
        self.d = self.d._replace(**display_update)

        if load is not None:
            self.load(load)
        else:
            self.loaded = False
            self.scene_loaded = False
            self.vox_grid_set_up = False
            self.vox_group_set_up = False
            self.hull_made = False
            self.voxelised = False
            self.dft = False
            self.barrier_built = False
            self.rooms_solidified = False
            self.rooms_formed = False

            self.edge_voxels = False
            self.adjacencies_determined = False

    def proceed_with_barrier_building_stage(self):

        check1 = (self.s.barriers_to_build > 0)
        check2 = self.voxelised

        if check1 and check2:
            return True
        else:
            return False

    def save(self, folder_path, timings=None):

        progress_dict = {"loaded": self.loaded,
                         "scene_loaded": self.scene_loaded,

                         "vox_grid_set_up": self.vox_grid_set_up,
                         "self.vox_group_set_up": self.vox_group_set_up,
                         "hull_made": self.hull_made,
                         "voxelised": self.voxelised,
                         "dft": self.dft,
                         "barrier_built": self.barrier_built,
                         "rooms_solidified": self.rooms_solidified,
                         "rooms_formed": self.rooms_formed,

                         "edge_voxels": self.edge_voxels,
                         "adjacencies_determined": self.adjacencies_determined,
                         }

        progress_dict2 = {"settings": self.s._asdict(),
                          "show_settings": self.d._asdict(),
                          }

        progress_path = folder_path / (folder_path.stem + "_progress.json")
        progress_path2 = folder_path / (folder_path.stem + "_progress.json")
        timings_path = folder_path / (folder_path.stem + "_timings.json")
        rooms_path = folder_path / (folder_path.stem + "_roomcount.json")

        with progress_path.open('w') as f:
            json.dump(progress_dict, f, indent=4)

        with progress_path2.open('w') as f:
            json.dump(progress_dict2, f, indent=4)
        try:
            with rooms_path.open('w') as f:
                json.dump(self.room_info, f, indent=4)
        except:
            pass

        with timings_path.open('w') as f:
            json.dump(timings.asdict(), f, indent=4)  # timings not namedtuple

    def load(self, folder_path):

        progress_path = folder_path / (folder_path.stem + "_progress.json")

        with progress_path.open('rb') as f:
            progress_dict = json.load(f)

        self.loaded = True
        self.scene_loaded = progress_dict.get("scene_loaded", False)
        self.vox_grid_set_up = progress_dict.get("vox_grid_set_up ", False)
        self.vox_group_set_up = progress_dict.get("vox_group_set_up ", False)
        self.hull_made = progress_dict.get("hull_made", False)
        self.voxelised = progress_dict.get("voxelised", False)
        self.dft = progress_dict.get("dft", False)
        self.barrier_built = progress_dict.get("barrier_built", False)
        self.edge_voxels = progress_dict.get("edge_voxels", False)
