# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

import pythonocc.Display.LegendGui as SimpleGui
import modules.display_room as dr

# Logging
from utils.logging_env import setup_logging, logging
log = logging.getLogger(__name__)
setup_logging(log)


def apply_colour(elements, rooms, special_cases=None):

    dr.apply_colour(elements, rooms, special_cases)


def displayer(scene, vox_group=None, progress=None, test=False):

    def print_feature(a, X, Y):

        for each in a:
            element = scene._elements.get(hash(each), None)
            if element is not None:
                print(element.get("guid", "No GUID"))
            else:
                print("No GUID")

    disp = {}
    display = SimpleGui.init_display(disp=disp, perspective=True, test=test)
    display.register_select_callback(print_feature)

    scene.init_display(display, disp, bounded_planes=True)
    scene.add_features_to_menu()
    view = {"view1": {"eye": progress.d.view_eye,
                      "centre": progress.d.view_centre,
                      "up": progress.d.view_up,
                      "scale": progress.d.view_scale,
                      "fov": progress.d.view_fov,
                      }
            }

    display.views.update(view)

    if vox_group:
        vox_group.display = display
        vox_group.disp = disp
        vox_group.add_grid_to_menu()
        vox_group.add_element_array_to_menu()
        vox_group.add_sources_to_menu()
        scene.add_hull_to_menu()
        vox_group.add_mask_to_menu()

        if vox_group.debug is not None:
            if vox_group.debug.sum():
                vox_group.add_debug_to_menu()

    if progress:
        if progress.dft:
            vox_group.add_dft_to_menu()
            vox_group.add_connect_to_menu()

            vox_group.add_dft_by_val_to_menu()
        log.info("Voxelised: %s" % progress.voxelised)
        log.info("Show rooms: %s" % progress.d.show_rooms)
        log.info("Rooms %s" % len(scene.rooms))
        if progress.voxelised and progress.d.show_rooms:
            if scene.rooms:
                room_display = dr.Displayed_Rooms(display,
                                                  disp,
                                                  scene.elements,
                                                  scene.rooms,
                                                  scene.barriers)
                room_display.display_rooms()

    display.FitAll()

    if progress:
        if progress.d.show_dft:
            vox_group.show_dft()

    disp['start']()

    scene.internals = []
    for each in scene.rooms:
        if scene._elements[scene.rooms[each]].get('visible', False):
            scene.internals.append(each)
