# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

from itertools import product

import numpy as np
from skimage.morphology import binary_dilation


def check_size_fits(offset, size, parent_size):

    if offset == 0:
        m = 0
    else:
        m = 1

    if offset + size >= parent_size:
        p = 0
    else:
        p = 1

    return m, p


def grow_barrier_array(a,
                       x_offset=0,
                       y_offset=0,
                       z_offset=0,
                       parent_sx=None,
                       parent_sy=None,
                       parent_sz=None,
                       diag=True,
                       hollow=True):

    a = a.astype(np.int32)
    sx, sy, sz = a.shape
    if parent_sx is not None:
        xm, xp = check_size_fits(x_offset, sx, parent_sx)
        ym, yp = check_size_fits(y_offset, sy, parent_sy)
        zm, zp = check_size_fits(z_offset, sz, parent_sz)
    else:
        xm, xp = 1, 1
        ym, yp = 1, 1
        zm, zp = 1, 1

    b = np.zeros((sx + xm + xp, sy + ym + yp, sz + zm + zp), np.int32)
    b[xm:sx + xm, ym:sy + ym, zm:sz + zm] = a

    if diag:
        d = np.ones((3, 3, 3), np.int32)
        c = binary_dilation(b, d)
    else:
        c = binary_dilation(b)

    if xm:
        x_offset -= 1
    if ym:
        y_offset -= 1
    if zm:
        z_offset -= 1

    # Algorithm assumes that barrier is contained within exisiting voxel array
    # TODO - trim dilated barrier if above assumption if not reliable
    if hollow:
        c = c - b

    return c, (x_offset, y_offset, z_offset)


def amend_mask(mask, dilated_barrier, off):

    sx, sy, sz, = dilated_barrier.shape
    for x, y, z in product(range(sx), range(sy), range(sz)):
        if dilated_barrier[x, y, z]:
            mask[x + off[0], y + off[1], z + off[2]] = 3


if __name__ == "__main__":

    parent = np.ones((5, 5, 5), np.int32)
    offset = (2, 2, 2)
    a = np.ones((1, 1, 1), np.int32)
    x_offset, y_offset, z_offset = offset

    c, new_offset = grow_barrier_array(a, x_offset, y_offset, z_offset,
                                       parent.shape[0],
                                       parent.shape[2],
                                       parent.shape[2],
                                       )
    print(c)
    print(new_offset)

    amend_mask(parent, c, new_offset)

    print(parent)
