# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

import numpy as np
from scipy.spatial import ConvexHull

from OCC.Core.BRep import BRep_Tool
import OCC.Core.BRepBuilderAPI as brep
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakePolygon as MakePolygon
from OCC.Core.gp import gp_Pnt
import OCC.Core.TopoDS as tp
from OCCUtils.Topology import Topo

import modules.analyse_settings as analyse_settings


def hull(scene_elements):

    # element = scene_elements.__iter__().__next__()
    # first = scene_elements[element]['shape']
    brt = BRep_Tool()
    points = []
    for each in scene_elements:
        topo = Topo(scene_elements[each]['shape'])
        points.extend([brt.Pnt(x).Coord() for x in topo.vertices()])

    points = np.array(points)

    hull = ConvexHull(points)
    faces = []
    mkShell = brep.BRepBuilderAPI_Sewing()

    for simplex in hull.simplices:
        simplex_list = simplex.tolist()
        face = points[simplex_list]
        verts = [gp_Pnt(*each_face) for each_face in face]
        poly = MakePolygon(verts[0], verts[1], verts[2], True)
        face = brep.BRepBuilderAPI_MakeFace(poly.Wire()).Face()
        faces.append(face)
        mkShell.Add(face)
    mkShell.Perform()
    shell = tp.topods.Shell(mkShell.SewedShape())
    solid = brep.BRepBuilderAPI_MakeSolid(shell).Solid()

    return solid


def main():

    settings = analyse_settings.load_settings(
        3,
        3,
        3,
        3)

    global scene_elements
    disp = {}
    scene_elements = {}

    # walls = TwoRooms(scene_elements)
    Scene(settings, scene_elements)

    hull_shape = hull(scene_elements)

    display = SimpleGui.init_display("qt-pyqt5", disp=disp)
    blue = qu.Quantity_Color(qu.Quantity_NOC_BLUE1)
    glass = gr.Graphic3d_MaterialAspect(gr.Graphic3d_NOM_GLASS)
    display.DisplayShape(hull_shape, color=blue, material=glass)
    display.FitAll()
    disp['start']()


if (__name__ == '__main__'):

    from pythonocc.Display import SimpleGui
    from modules.scene import Scene
    import OCC.Quantity as qu
    import OCC.Graphic3d as gr
    main()
