# python3
# -*- coding: utf-8 -*-

# Copyright Gareth Boyes, 2021

import OCC.Core.BRepBndLib as blib
import OCC.Core.Bnd as bnd
import OCC.Core.BRepBuilderAPI as brep
import OCC.Core.BRepPrimAPI as bp
import OCC.Core.gp as gp

# Logging
from utils.logging_env import setup_logging, logging
log = logging.getLogger(__name__)
setup_logging(log)


class Empty(object):

    def __init__(self, scene_elements, trsf_G2V=None,):

        self.scene_elements = scene_elements
        # self.

        if trsf_G2V is not None:
            raise Exception

        scene_dict = {}

        self.process_scenario(scene_dict)

    def process_scenario(self, scene_dict):

        walls = {}

        for each in scene_dict:
            bb = bnd.Bnd_Box()
            blib.brepbndlib_Add(scene_dict[each], bb)
            hash_ = scene_dict[each].__hash__()

            cornermin = bb.CornerMin().Coord()
            cornermax = bb.CornerMax().Coord()
            corner_tuple = (*cornermin, *cornermax)

            walls[hash_] = {'shape': scene_dict[each],
                            'name': each,
                            'ais': None,
                            'bb': bb,
                            'bbxyz': corner_tuple}

            # self._elements.append(hash_)

        self.scene_elements.update(walls)


class TwoRooms(object):

    def __init__(self, scene_elements, scenario=1, trsf_G2V=None,):

        self.elements = []

        self.scene_elements = scene_elements
        if scenario == 1:
            self.scenario_1()
        if scenario == 2:
            self.scenario_2()

    def scenario_1(self):

        w1 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(-10, -10, -10),
                                    gp.gp_Pnt(-9, 10, 10)).Solid()  # west
        w2 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(-9, 9, -10),
                                    gp.gp_Pnt(28, 10, 10)).Solid()  # north
        w3 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(28, -10, -10),
                                    gp.gp_Pnt(29, 10, 10)).Solid()  # east
        w4 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(-9, -10, -10),
                                    gp.gp_Pnt(28, -9, 10)).Solid()  # south
        w5 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(-9, -9, -10),
                                    gp.gp_Pnt(28, 9, -9)).Solid()  # bottom
        w6 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(-9, -9, 9),
                                    gp.gp_Pnt(28, 9, 10)).Solid()  # top
        w9 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(8, -9, -9),
                                    gp.gp_Pnt(9, -3, 9)).Solid()  # middle1
        w10 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(11, 2, -9),
                                     gp.gp_Pnt(12, 9, 9)).Solid()  # middle2

        scene_dict = {'west': w1,
                      'north': w2,
                      'east': w3,
                      'south': w4,
                      'bottom': w5,
                      'top': w6,
                      'middle1': w9,
                      'middle2': w10}

        self.process_scenario(scene_dict)

    def scenario_2(self):

        wall_width = 1
        height = 6
        room_width = 12

        offset = 0.5

        y_offset = 0
        gap = 1.1

        a = - (height / 2) - 1
        b = (height / 2) + 1
        c = - (height / 2)
        d = (height / 2)

        x1 = - room_width / 2 - wall_width
        x2 = - room_width / 2
        x3 = room_width * 1.5 + wall_width

        x4 = room_width / 2 - offset
        x5 = x4 + wall_width
        x6 = room_width * 1.5 + wall_width * 2

        x7 = room_width / 2
        x8 = x7 + wall_width

        y1 = - room_width / 2 - wall_width
        y2 = - room_width / 2
        y3 = room_width / 2
        y4 = room_width / 2 + wall_width

        y5 = - y_offset - gap / 2
        y6 = - y_offset + gap / 2

        w1 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x1, y1, a),
                                    gp.gp_Pnt(x2, y4, b)).Solid()  # west
        w2 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y3, a),
                                    gp.gp_Pnt(x3, y4, b)).Solid()  # north
        w3 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x3, y1, a),
                                    gp.gp_Pnt(x6, y4, b)).Solid()  # east
        w4 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y1, a),
                                    gp.gp_Pnt(x3, y2, b)).Solid()  # south
        w5 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y2, a),
                                    gp.gp_Pnt(x3, y3, c)).Solid()  # bottom
        w6 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y2, d),
                                    gp.gp_Pnt(x3, y3, b)).Solid()  # top
        w9 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x4, y2, c),
                                    gp.gp_Pnt(x5, y5, d)).Solid()  # middle1
        w10 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x7, y6, c),
                                     gp.gp_Pnt(x8, y3, d)).Solid()  # middle2

        scene_dict = {'west': w1,
                      'north': w2,
                      'east': w3,
                      'south': w4,
                      'bottom': w5,
                      'top': w6,
                      'middle1': w9,
                      'middle2': w10}

        self.process_scenario(scene_dict)

    def process_scenario(self, scene_dict):

        walls = {}

        for each in scene_dict:
            bb = bnd.Bnd_Box()
            blib.brepbndlib_Add(scene_dict[each], bb)
            hash_ = scene_dict[each].__hash__()

            cornermin = bb.CornerMin().Coord()
            cornermax = bb.CornerMax().Coord()
            corner_tuple = (*cornermin, *cornermax)

            walls[hash_] = {'shape': scene_dict[each],
                            'name': each,
                            'ais': None,
                            'bb': bb,
                            'bbxyz': corner_tuple}

            self.elements.append(hash_)

        self.scene_elements.update(walls)


class Tworoomset(object):

    def __init__(self, settings, scene_elements, trsf_G2V=None):

        self.elements = []
        self.scene_elements = scene_elements

        if trsf_G2V is not None:
            raise Exception

        wall_width = settings['wall_width']
        height = settings['height']
        room_width = settings['room_width']

        wall_offset = settings['wall_offset']

        wall_y_offset = settings['wall_y_offset']
        gap = settings['gap']
        two_gaps = settings['two_gaps']

        x = settings['x_offset']
        y = settings['y_offset']
        z = settings['z_offset']

        a = - (height / 2) - 1 + z
        b = (height / 2) + 1 + z
        c = - (height / 2) + z
        d = (height / 2) + z

        x1 = - room_width / 2 - wall_width + x
        x2 = - room_width / 2 + x
        x3 = room_width * 1.5 + wall_width + x

        x4 = room_width / 2 - wall_offset + x
        x5 = x4 + wall_width + x
        x6 = room_width * 1.5 + wall_width * 2 + x

        x7 = room_width / 2 + x
        x8 = x7 + wall_width + x

        y1 = - room_width / 2 - wall_width + y
        y2 = - room_width / 2 + y
        y3 = room_width / 2 + y
        y4 = room_width / 2 + wall_width + y

        y5 = - wall_y_offset - gap / 2 + y
        y6 = - wall_y_offset + gap / 2 + y

        y7 = - wall_y_offset + gap / 2 + gap + y
        y8 = - wall_y_offset + gap / 2 + gap * 2 + y

        w1 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x1, y1, a),
                                    gp.gp_Pnt(x2, y4, b)).Solid()  # west
        w2 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y3, a),
                                    gp.gp_Pnt(x3, y4, b)).Solid()  # north
        w3 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x3, y1, a),
                                    gp.gp_Pnt(x6, y4, b)).Solid()  # east
        w4 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y1, a),
                                    gp.gp_Pnt(x3, y2, b)).Solid()  # south
        w5 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y2, a),
                                    gp.gp_Pnt(x3, y3, c)).Solid()  # bottom
        w6 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x2, y2, d),
                                    gp.gp_Pnt(x3, y3, b)).Solid()  # top

        w9 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x4, y2, c),
                                    gp.gp_Pnt(x5, y5, d)).Solid()  # middle1
        if not two_gaps:
            w10 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x7, y6, c),
                                         gp.gp_Pnt(x8, y3, d)).Solid()  # mid2
        else:
            w10 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x7, y6, c),
                                         gp.gp_Pnt(x8, y7, d)).Solid()  # mid2
            w11 = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x7, y8, c),
                                         gp.gp_Pnt(x8, y3, d)).Solid()  # mid3

        scene_dict = {'west': w1,
                      'north': w2,
                      'east': w3,
                      'south': w4,
                      'bottom': w5,
                      'top': w6,
                      'middle1': w9,
                      'middle2': w10}

        if two_gaps:
            scene_dict['middle3'] = w11

        self.process_scenario(scene_dict)

    def process_scenario(self, scene_dict):

        walls = {}

        for each in scene_dict:
            bb = bnd.Bnd_Box()

            blib.brepbndlib_Add(scene_dict[each], bb)
            hash_ = scene_dict[each].__hash__()

            cornermin = bb.CornerMin().Coord()
            cornermax = bb.CornerMax().Coord()
            corner_tuple = (*cornermin, *cornermax)

            walls[hash_] = {'shape': scene_dict[each],
                            'name': each,
                            'ais': None,
                            'bb': bb,
                            'bbxyz': corner_tuple
                            }

            self.elements.append(hash_)

        self.scene_elements.update(walls)


class Vertex(object):

    def __init__(self, scene_elements, trsf_G2V=None, scenario=1):

        self.elements = []

        if trsf_G2V is None:
            raise Exception

        self.scene_elements = scene_elements
        if scenario == 1:
            self.scenario_1()

    def scenario_1(self):

        v1 = brep.BRepBuilderAPI_MakeVertex(gp.gp_Pnt(19.5,
                                                      19.5, 19.5)).Vertex()
        scene_dict = {'point': v1}
        self.process_scenario(scene_dict)

    def process_scenario(self, scene_dict):

        verts = {}

        for each in scene_dict:
            bb = bnd.Bnd_Box()
            blib.brepbndlib_Add(scene_dict[each], bb)
            hash_ = scene_dict[each].__hash__()

            cornermin = bb.CornerMin().Coord()
            cornermax = bb.CornerMax().Coord()
            corner_tuple = (*cornermin, *cornermax)

            verts[hash_] = {'shape': scene_dict[each],
                            'name': each,
                            'ais': None,
                            'bb': bb,
                            'bbxyz': corner_tuple}

            self.elements.append(hash_)

        self.scene_elements.update(verts)


class Illustration(object):

    def __init__(self, scene_elements):

        self.elements = []
        self.scene_elements = scene_elements
        scene_dict = {}
        left = 1 + 1
        bottom = 1 + 1

        top = 19 + 1
        right = 23 + 1

        halfway_up = 12 + 1
        middle = left + round((right - left) / 2)
        gap_a = left + round((right - left) / 4)
        gap_b = left + round(3 * (right - left) / 4)
        gap_c = bottom + round((halfway_up - bottom) / 2)

        gap_width = 2

        self.data = ((left, bottom, left, top),
                     (left, top, right, top),
                     (right, top, right, bottom),
                     (left, bottom, right, bottom),
                     (left, halfway_up, gap_a - gap_width, halfway_up),
                     (gap_a + gap_width, halfway_up,
                      gap_b - gap_width, halfway_up),
                     (gap_b + gap_width, halfway_up, right, halfway_up),
                     (middle, bottom, middle, gap_c - gap_width),
                     (middle, gap_c + gap_width, middle, halfway_up))

        self.width = 0.25
        width = self.width
        floor = 1
        ceiling = 11

        for n, each in enumerate(self.data):

            x0, y0, x1, y1 = each
            solid = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(x0 - width,
                                                     y0 - width,
                                                     floor),
                                           gp.gp_Pnt(x1 + width,
                                                     y1 + width,
                                                     ceiling)).Solid()

            scene_dict[n] = solid

        floor = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(left - width,
                                                 bottom - width,
                                                 floor - width),
                                       gp.gp_Pnt(right + width,
                                                 top + width,
                                                 floor)).Solid()

        ceiling = bp.BRepPrimAPI_MakeBox(gp.gp_Pnt(left - width,
                                                   bottom - width,
                                                   ceiling),
                                         gp.gp_Pnt(right + width,
                                                   top + width,
                                                   ceiling + width)).Solid()

        scene_dict["floor"] = floor
        scene_dict["ceiling"] = ceiling

        self.process_scenario(scene_dict)

    def process_scenario(self, scene_dict):

        walls = {}

        for each in scene_dict:
            bb = bnd.Bnd_Box()

            blib.brepbndlib_Add(scene_dict[each], bb)
            hash_ = hash(scene_dict[each])

            cornermin = bb.CornerMin().Coord()
            cornermax = bb.CornerMax().Coord()
            corner_tuple = (*cornermin, *cornermax)

            walls[hash_] = {'shape': scene_dict[each],
                            'name': each,
                            'ais': None,
                            'bb': None,
                            'bbxyz': corner_tuple
                            }
            self.elements.append(hash_)

        self.scene_elements.update(walls)

    def polygons(self):

        poly = []
        width = self.width
        for x0, y0, x1, y1 in self.data:

            polygon = ((x0 - width + 0.5, y0 - width + 0.5),
                       (x1 + width + 0.5, y0 - width + 0.5),
                       (x1 + width + 0.5, y1 + width + 0.5),
                       (x0 - width + 0.5, y1 + width + 0.5))

            poly.append(polygon)

        return poly
