from distutils.core import setup, Extension
from Cython.Build import cythonize

import getpass
import sys
import pathlib
import shutil

home = pathlib.Path.home()

def compile(py_script_folder, py_script_file):
    
    project_path = home / "control" / "bitbucket" / "phd"
    destination = project_path / "cython_builds"
      
    py_script_path = project_path / py_script_folder / py_script_file
    
    renamed = py_script_file.replace("_py.py", ".py")
    renamed_path = project_path/ py_script_folder / renamed

    shutil.copy(py_script_path, renamed_path)
    
    cython_script = renamed_path.as_posix()
    script_name = renamed_path.stem
    cd = {'language_level':'3'}
    
    setup(ext_modules=cythonize(cython_script, compiler_directives=cd),
          script_name=script_name,
          script_args=['build_ext', '--build-lib', destination])  
    
    renamed_path.unlink()  

if __name__ == "__main__":
    
    cython_files = [("voxel_creator_", "voxel_creator_py.py"),
                    ("manifold", "check_manifold_py.py"),
                    ("neighbour_skimage", "neighbour_ski_ig_py.py")]
    
    for each in cython_files:
        
        py_script_folder = each[0]
        py_script_file = each[1]
    
        compile(py_script_folder, py_script_file)    
    
 