# cython3
import cython
import numpy as np

import OCC.Core.Graphic3d as gr
import OCC.Core.Quantity as qu
import OCC.Core.gp as gp

import random

import OCC.Core.BRepBuilderAPI as brep
import OCC.Core.TopoDS as tp
from OCC.Core.TopExp import TopExp_Explorer
from OCC.Core.TopoDS import TopoDS_Compound, TopoDS_Wire, TopoDS_Face
from OCC.Core.BRep import BRep_Builder


#&
@cython.ccall
#&
def transform (x1: cython.double,
               y1: cython.double,
               z1: cython.double,
               matrix: cython.double[:,::1]):
    
##cpdef transform(double x1, double y1, double z1, double [:,::1] matrix):
#&
    x: cython.double = 0
    y: cython.double = 0
    z: cython.double = 0
    
    x +=  np.double(matrix[0,0]) * x1
    x += y1 * np.double(matrix[0,1] )
    x += z1 * np.double(matrix[0,2] )
    x +=  np.double(matrix[3,0]) 
    
    y += x1 *  np.double(matrix[1,0] )
    y += y1 *  np.double(matrix[1,1] )
    y += z1 *  np.double(matrix[1,2] )
    y +=  np.double(matrix[3,1]) 
    
    z += x1 * np.double(matrix[2,0] )
    z += y1 * np.double(matrix[2,1] )
    z += z1 * np.double(matrix[2,2] )
    z +=  np.double(matrix[3,2]) 
    
    return (x,y,z)


@cython.ccall
#&
def iterate2(mv : cython.int[:,:,::1],
             mv2: cython.int[:,:,::1],
             group,
             trsf,
             min_val : cython.int,
             val : cython.int) -> cython.bint:
##def iterate2(int [:,:,::1] mv, int [:,:,::1] mv2, group, trsf, int min_val, int val):

##  cdef int x, y, z, sx, sy, sz
##  cdef bint t,b,l,r,f,back
    x : cython.int
    y : cython.int
    z : cython.int
    sx : cython.int
    sy : cython.int
    sz : cython.int
    
    t : cython.bint
    b : cython.bint
    l : cython.bint
    r : cython.bint
    f : cython.bint
    back : cython.bint
    
    sx = mv.shape[0]
    sy = mv.shape[1]
    sz = mv.shape[2]
    
    min_val -= 1 
        
    for x in range(sx):
        for y in range (sy):
            for z in range (sz):
                t = 0
                b = 0
                r = 0
                l = 0
                f = 0
                back = 0
        
                if min_val < mv[x,y,z] <= val and mv[x,y,z] != 0 :
                
                    if mv2[x,y,z] == 1:
                        continue
                    
                    if x == 0:
                        l = True
                        
                    elif mv2[x-1,y,z] == 1:
                        l = True                  
                        
                    elif mv[x-1,y,z] > val or mv[x-1,y,z] <= min_val:
                        l = True
                
                        
                    if y == 0:
                        f = True
                    
                    elif mv2[x,y-1,z] == 1:
                        f = True                            
                        
                    elif  mv[x,y-1,z] > val or mv[x,y-1,z] <= min_val:
                        f = True
        
                    if z == 0:
                        b = True
                    
                    elif mv2[x,y,z-1] == 1:
                        b = True                            
                        
                    elif  mv[x,y,z-1] > val or mv[x,y,z-1] <= min_val:
                        b = True
    
                    if x==sx-1:
                        r = True 
                    
                    elif mv2[x+1,y,z] == 1:
                        r = True                           
                        
                    elif  mv[x+1,y,z] > val or mv[x+1,y,z] <= min_val:
                        r = True
                        
                    if y==sy-1:
                        back = True
                        
                    elif mv2[x,y+1,z] == 1:
                        back = True                      
                        
                    elif  mv[x,y+1,z] > val or mv[x,y+1,z] <= min_val:
                        back = True
        
                    if z==sz-1:
                        t = True
                    
                    elif mv2[x,y,z+1] == 1:
                        t = True
                        
                    elif  mv[x,y,z+1] > val or mv[x,y,z+1] <= min_val:
                        t = True
                                       
                    display_v(group, trsf, x,y,z, t,b,l,r,f,back, mv[x,y,z])
        
    return True

#&
@cython.ccall
def display_v(group,
              trsf,
              x : cython.double,
              y : cython.double,
              z : cython.double,
              top : cython.bint,
              bottom : cython.bint,
              left : cython.bint,
              right : cython.bint,
              front : cython.bint,
              back : cython.bint,
              val: cython.int,
              ) -> cython.bint:
    
##def display_v(group, trsf, double x, double y, double z, bint top, bint bottom,\
##bint left, bint right, bint front, bint back, int val):


##  cdef:
##      double hs
##      double xm,xp
##      double ym,yp
##      double zm,zp
        
##      double Ax,Ay,Az,Bx,By,Bz,Cx,Cy,Cz,Dx,Dy,Dz
##      double Ex,Ey,Ez,Fx,Fy,Fz,Gx,Gy,Gz,Hx,Hy,Hz

     
    hs : cython.double
    xm : cython.double
    xp : cython.double
    ym : cython.double
    yp : cython.double
    zm : cython.double
    zp : cython.double

    Ax : cython.double
    Ay : cython.double
    Az : cython.double
    
    Bx : cython.double
    By : cython.double
    Bz : cython.double

    Cx : cython.double
    Cy : cython.double
    Cz : cython.double
    
    Dx : cython.double
    Dy : cython.double
    Dz : cython.double
    
    Ex : cython.double
    Ey : cython.double
    Ez : cython.double
    
    Fx : cython.double
    Fy : cython.double
    Fz : cython.double
    
    Gx : cython.double
    Gy : cython.double
    Gz : cython.double
    
    Hx : cython.double
    Hy : cython.double
    Hz : cython.double
    
    red : cython.double
    green: cython.double
    blue : cython.double
    
    col : cython.int
    
##&
    #bottomleft = bottomright =False
##&      
    #bf = bb = tl = tr = tf = tb = fl = fr = False
##& 
    #backright = backleft = False
    
##  cdef:      
##      bint bottomleft = False
##      bint bottomright = False
##      bint bf = False
##      bint bb = False
##      bint tl = False
##      bint tr = False
##      bint tf = False
##      bint tb = False       
##      bint fl = False
##      bint fr = False
##      bint backright = False
##      bint backleft = False
##      double red, green, blue

    bottomleft : cython.bint = False
    bottomright : cython.bint = False
    bf : cython.bint= False
    bb : cython.bint= False
    tl : cython.bint= False
    tr : cython.bint= False
    tf : cython.bint= False
    tb : cython.bint= False       
    fl : cython.bint= False
    fr : cython.bint= False
    backright : cython.bint= False
    backleft : cython.bint= False

    col = val%10

    if val == 0:
        red = 0
        green = 0
        blue = 1  
    
    elif val == 1:
        red = 0.67843097448349  
        green = 0.8470590114593506
        blue = 0.9019610285758972
    
    elif col == 0:
        red = 0.619607843137255
        green = 0.00392156862745098
        blue = 0.258823529411765
    elif col==1:
        red = 0.835294117647059
        green = 0.243137254901961
        blue = 0.309803921568627

    elif col ==2:
        red = 0.956862745098039
        green = 0.427450980392157
        blue = 0.262745098039216
        
    elif col ==3:
        red = 0.992156862745098
        green = 0.682352941176471
        blue = 0.380392156862745
    elif col ==4:
        red = 0.996078431372549
        green = 0.87843137254902
        blue = 0.545098039215686
    elif col ==5:
        red = 0.901960784313726
        green = 0.96078431372549
        blue = 0.596078431372549
    elif col ==6:
        red = 0.670588235294118
        green = 0.866666666666667
        blue = 0.643137254901961
    elif col ==7:
        red = 0.4
        green = 0.76078431372549
        blue = 0.647058823529412
    elif col ==8:
        red = 0.196078431372549
        green = 0.533333333333333
        blue = 0.741176470588235
    elif col ==9:
        red = 0.368627450980392
        green = 0.309803921568627
        blue = 0.635294117647059
        
    elif col == 10:
        red = 0.9
        green = 0.9
        blue = 0.9
    
         
    hs = 0.5
    xm = x-hs
    xp = x+hs
    ym = y-hs
    yp = y+hs
    zm = z-hs
    zp = z+hs
    
    Ax, Ay, Az = transform(xm,ym,zm,trsf)
    Bx, By, Bz = transform(xp,ym,zm,trsf)
    Cx, Cy, Cz = transform(xp,yp,zm,trsf)
    Dx, Dy, Dz = transform(xm,yp,zm,trsf)
    
    Ex, Ey, Ez = transform(xm,ym,zp,trsf)
    Fx, Fy, Fz = transform(xp,ym,zp,trsf)
    Gx, Gy, Gz = transform(xp,yp,zp,trsf)
    Hx, Hy, Hz = transform(xm,yp,zp,trsf)
    
    if bottom:
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)
        poly.AddVertex(Dx,Dy,Dz)   #D,
        poly.AddVertex(Cx,Cy,Cz)   #C,
        poly.AddVertex(Bx,By,Bz)   #B,
        poly.AddVertex(Ax, Ay, Az) #A 
        group.AddPrimitiveArray(poly)
        
        bottomleft = True
        bottomright = True
        bf =  True
        bb =  True
         
    if top:
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)

        poly.AddVertex(Ex,Ey,Ez) #E, 
        poly.AddVertex(Fx,Fy,Fz) #F,
        poly.AddVertex(Gx,Gy,Gz) #G,
        poly.AddVertex(Hx,Hy,Hz) #H,
        group.AddPrimitiveArray(poly) 
        
        tl =  True
        tr =  True
        tf =  True
        tb =  True
             
 
    if front:    
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)
        poly.AddVertex(Ax, Ay, Az)
        poly.AddVertex(Bx,By,Bz)
        poly.AddVertex(Fx,Fy,Fz)
        poly.AddVertex(Ex,Ey,Ez)
        group.AddPrimitiveArray(poly) 
             
             
        fl =  True
        fr =  True
        tf =  True
        bf =  True
    
    if right:
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)
        poly.AddVertex(Bx,By,Bz) #B
        poly.AddVertex(Cx,Cy,Cz) #C
        poly.AddVertex(Gx,Gy,Gz) #G
        poly.AddVertex(Fx,Fy,Fz) #F
        group.AddPrimitiveArray(poly) 
             
        backright =  True
        bottomright = True
        fr = True
        tr = True
        
    if back:      
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)
        poly.AddVertex(Cx,Cy,Cz) #C
        poly.AddVertex(Dx,Dy,Dz) #D
        poly.AddVertex(Hx,Hy,Hz) #H
        poly.AddVertex(Gx,Gy,Gz) #G
        group.AddPrimitiveArray(poly) 
             
        backleft  = True
        backright = True
        bf = True
        bb =  True
        
    if left:
        poly = gr.Graphic3d_ArrayOfPolygons(4,2,0,False, False, True)
        
        poly.AddBound(4)
        poly.SetBoundColor(1, red,green,blue)
        poly.AddVertex(Dx,Dy,Dz) #D
        poly.AddVertex(Ax,Ay,Az) #A
        poly.AddVertex(Ex,Ey,Ez) #E
        poly.AddVertex(Hx,Hy,Hz) #H
        group.AddPrimitiveArray(poly) 
             
        backleft  =  True
        tl = True
        bottomleft =  True
        fl = True
        
    if not right:
        bottomright = False
        fr = False
        backright = False
        tr = False

    if not top:
        tr = False
        tl = False
        tf = False
        tb = False
        
    if not back:
        
        bb = False
        backright = False
        backleft = False
        tb = False
        
    if bb: 
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Dx,Dy,Dz)
        line.AddVertex(Cx,Cy,Cz)
        group.AddPrimitiveArray(line) 
        
    if bottomright:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Cx,Cy,Cz)
        line.AddVertex(Bx,By,Bz)
        group.AddPrimitiveArray(line)
        
    if bf:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Bx,By,Bz)
        line.AddVertex(Ax,Ay,Az)
        group.AddPrimitiveArray(line)
        
    if bottomleft:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Ax, Ay, Az)
        line.AddVertex(Dx,Dy,Dz)
        group.AddPrimitiveArray(line) 

    if tb: 
             
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Hx,Hy,Hz)
        line.AddVertex(Gx,Gy,Gz)
        group.AddPrimitiveArray(line) 
        
    if tr:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Gx,Gy,Gz)
        line.AddVertex(Fx,Fy,Fz)
        group.AddPrimitiveArray(line) 
        
    if tf:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Fx,Fy,Fz)
        line.AddVertex(Ex,Ey,Ez)
        group.AddPrimitiveArray(line)
        
    if tl:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Ex,Ey,Ez)
        line.AddVertex(Hx,Hy,Hz)
        group.AddPrimitiveArray(line) 
        
    if backright:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Cx,Cy,Cz)
        line.AddVertex(Gx,Gy,Gz)
        group.AddPrimitiveArray(line) 
        
    if backleft:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Dx,Dy,Dz)
        line.AddVertex(Hx,Hy,Hz)
        group.AddPrimitiveArray(line) 
        
    if fl:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Ax, Ay, Az)
        line.AddVertex(Ex,Ey,Ez)
        group.AddPrimitiveArray(line) 
        
    if fr:
        
        line = gr.Graphic3d_ArrayOfPolylines(2)
        line.AddVertex(Bx,By,Bz)
        line.AddVertex(Fx,Fy,Fz)
        group.AddPrimitiveArray(line)
        
    return True

#&
def iterate3(mv, offset, trsf, val):
##def iterate3(int [:,:,::1] mv, list offset, trsf, int val): 
    
##  cdef int x, y, z, dx, dy, dz, sx, sy, N3
    
    x_offset, y_offset, z_offset = offset
    
    sx = mv.shape[0]
    sy = mv.shape[1]
    sz = mv.shape[2]
    
##  cdef bint t,b,l,r,f,back

    faces = []
        
    for x in range(sx):
        for y in range (sy):
            for z in range (sz):
                
                t = 0
                b = 0
                r = 0
                l = 0
                f = 0
                back = 0
        
                
                if mv[x,y,z] == val:
                
                    if x == 0:
                        l = True
                    elif mv[x-1,y,z] != val:
                        l = True
                        
                    if y == 0:
                        f = True
                    elif mv[x,y-1,z] != val  :
                        f = True
        
                    if z == 0:
                        b = True
                    elif mv[x,y,z-1] != val:
                        b = True
    
                    if x==sx-1:
                        r = True
                    elif mv[x+1,y,z] != val:
                        r = True
                        
                    if y==sy-1:
                        back = True
                    elif mv[x,y+1,z] != val:
                        back= True
        
                    if z==sz-1:
                        t = True
                    elif mv[x,y,z+1] != val:
                        t = True
                                       
                    
                    faces.extend ( return_faces(trsf,
                                              x+x_offset,
                                              y+y_offset,
                                              z+z_offset,
                                              t,
                                              b,
                                              l,
                                              r,
                                              f,
                                              back))
    
                            
    mkShell = brep.BRepBuilderAPI_Sewing()
    
    for each_face in faces:            
        mkShell.Add(each_face)
    mkShell.Perform()

    sew = mkShell.SewedShape()

    
    if sew is None:
        return None

    elif sew.ShapeType() == 0:
        compound = tp.topods.Compound(sew)
        return compound
        #compound = []
        #exp = TopExp_Explorer(sew, 3)
        #while exp.More():    
            #compound.append(exp.Current())
            #exp.Next()
        #if compound[0].ShapeType() == 3:
            #shell = tp.topods.Shell(compound[0])
            
    elif sew.ShapeType() == 3:
        shell = tp.topods.Shell(sew)
        
        solid = brep.BRepBuilderAPI_MakeSolid(shell).Solid()  
    
        return solid
 
@cython.ccall
def make_faces(mv: cython.int [:,:,::1],
               offset : tuple,
               trsf: cython.double[:,::1],
               val: cython.int):

    ##  cdef int x, y, z, dx, dy, dz, sx, sy, N3
    x_offset: cython.int
    y_offset: cython.int
    z_offset: cython.int
    x_offset, y_offset, z_offset = offset
        
        
    sx: cython.int = mv.shape[0]
    sy: cython.int  = mv.shape[1]
    sz: cython.int = mv.shape[2]
        
    ##  cdef bint t,b,l,r,f,back
    
    x: cython.int
    y: cython.int
    z: cython.int
    
    t: cython.int
    b: cython.int
    l: cython.int
    r: cython.int
    f: cython.int
    back: cython.int
    
    
    faces: list = []
        
    for x in range(sx):
        for y in range (sy):
            for z in range (sz):
                
                t = 0
                b = 0
                r = 0
                l = 0
                f = 0
                back = 0
        
                
                if mv[x,y,z] == val:
                
                    if x == 0:
                        l = True
                    elif mv[x-1,y,z] != val:
                        l = True
                        
                    if y == 0:
                        f = True
                    elif mv[x,y-1,z] != val  :
                        f = True
        
                    if z == 0:
                        b = True
                    elif mv[x,y,z-1] != val:
                        b = True
    
                    if x==sx-1:
                        r = True
                    elif mv[x+1,y,z] != val:
                        r = True
                        
                    if y==sy-1:
                        back = True
                    elif mv[x,y+1,z] != val:
                        back= True
        
                    if z==sz-1:
                        t = True
                    elif mv[x,y,z+1] != val:
                        t = True
                                       
                    
                    faces.extend ( return_faces(trsf,
                                              x+x_offset,
                                              y+y_offset,
                                              z+z_offset,
                                              t,
                                              b,
                                              l,
                                              r,
                                              f,
                                              back))
    
                            
    compound = TopoDS_Compound()
    builder = BRep_Builder()
    builder.MakeCompound(compound)
    
    for each_face in faces:            
        builder.Add(compound, each_face)

    return compound


def return_faces (trsf: cython.double[:,::1],
                  x: cython.int,
                  y: cython.int,
                  z: cython.int,
                  top: cython.int,
                  bottom: cython.int,
                  left: cython.int,
                  right: cython.int,
                  front: cython.int,
                  back: cython.int):
    
##def return_faces(trsf, double x, double y, double z, bint top, bint bottom,\
##bint left, bint right, bint front, bint back):
    
##  cdef:
##      double hs
##      double xm,xp
##      double ym,yp
##      double zm,zp
        
##      double Ax,Ay,Az,Bx,By,Bz,Cx,Cy,Cz,Dx,Dy,Dz
##      double Ex,Ey,Ez,Fx,Fy,Fz,Gx,Gy,Gz,Hx,Hy,Hz
         
    bottomleft : cython.bint = False
    bottomright : cython.bint = False
    bf : cython.bint= False
    bb : cython.bint= False
    tl : cython.bint= False
    tr : cython.bint= False
    tf : cython.bint= False
    tb : cython.bint= False       
    fl : cython.bint= False
    fr : cython.bint= False
    backright : cython.bint= False
    backleft : cython.bint= False
    
    faces: list = []
         
    hs: cython.double = 0.5
    xm: cython.double  = x-hs
    xp: cython.double  = x+hs
    ym: cython.double  = y-hs
    yp: cython.double  = y+hs
    zm: cython.double  = z-hs
    zp: cython.double  = z+hs
    
    Ax: cython.double
    Ay: cython.double
    Az: cython.double
    Bx: cython.double
    By: cython.double
    Bz: cython.double
    Cx: cython.double
    Cy: cython.double
    Cz: cython.double
    Dx: cython.double
    Dy: cython.double
    Dz: cython.double
    Ex: cython.double
    Ey: cython.double
    Ez: cython.double
    Fx: cython.double
    Fy: cython.double
    Fz: cython.double
    Gx: cython.double
    Gy: cython.double
    Gz: cython.double
    Hx: cython.double
    Hy: cython.double
    Hz: cython.double
    
    Ax, Ay, Az = transform(xm,ym,zm,trsf) #A
    Bx, By, Bz = transform(xp,ym,zm,trsf)
    Cx, Cy, Cz = transform(xp,yp,zm,trsf)
    Dx, Dy, Dz = transform(xm,yp,zm,trsf)
    
    Ex, Ey, Ez = transform(xm,ym,zp,trsf)
    Fx, Fy, Fz = transform(xp,ym,zp,trsf)
    Gx, Gy, Gz = transform(xp,yp,zp,trsf)
    Hx, Hy, Hz = transform(xm,yp,zp,trsf) #H
    
    wire: TopoDS_Wire
    face: TopoDS_Face
    
    if bottom:
        
        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Dx,Dy,Dz),
                                        gp.gp_Pnt(Cx,Cy,Cz),
                                        gp.gp_Pnt(Bx,By,Bz),
                                        gp.gp_Pnt(Ax, Ay, Az),
                                        True).Wire()
                                        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)
             
    if top:
    
        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Ex,Ey,Ez),
                                        gp.gp_Pnt(Fx,Fy,Fz),
                                        gp.gp_Pnt(Gx,Gy,Gz),
                                        gp.gp_Pnt(Hx, Hy, Hz),
                                        True).Wire()
        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)
             
    if front:    
    
        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Ax,Ay,Az),
                                        gp.gp_Pnt(Bx,By,Bz),
                                        gp.gp_Pnt(Fx,Fy,Fz),
                                        gp.gp_Pnt(Ex,Ey,Ez),
                                        True).Wire()
                                        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)
    
    if right:
    
        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Bx,By,Bz),
                                        gp.gp_Pnt(Cx,Cy,Cz),
                                        gp.gp_Pnt(Gx,Gy,Gz),
                                        gp.gp_Pnt(Fx,Fy,Fz),
                                        True).Wire()
                                        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)

    if back:

        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Cx,Cy,Cz),
                                        gp.gp_Pnt(Dx,Dy,Dz),
                                        gp.gp_Pnt(Hx,Hy,Hz),
                                        gp.gp_Pnt(Gx,Gy,Gz),
                                        True).Wire()
                                        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)
        
    if left:
    
        wire = brep.BRepBuilderAPI_MakePolygon(gp.gp_Pnt(Dx,Dy,Dz),
                                        gp.gp_Pnt(Ax,Ay,Az),
                                        gp.gp_Pnt(Ex,Ey,Ez),
                                        gp.gp_Pnt(Hx,Hy,Hz),
                                        True).Wire()
                                        
        face = brep.BRepBuilderAPI_MakeFace(wire).Face()
        faces.append(face)
    
    return faces

def display_grid(group, trsf, sx, sy, sz):
    
    for z in range(sz+1):
        for y in range(sy+1):
            line = gr.Graphic3d_ArrayOfPolylines(2)
            line.AddVertex(*transform(-0.5, y-0.5, z-0.5, trsf))
            line.AddVertex(*transform(sx-0.5, y-0.5, z-0.5, trsf))
            group.AddPrimitiveArray(line)  
    
    for y in range(sy+1):
        for x in range(sx+1):
            line = gr.Graphic3d_ArrayOfPolylines(2)
            line.AddVertex(*transform(x-0.5, y-0.5,  -0.5, trsf))
            line.AddVertex(*transform(x-0.5, y-0.5, sz-0.5, trsf))
            group.AddPrimitiveArray(line)      

    for x in range(sx+1):
        for z in range(sz+1):
            line = gr.Graphic3d_ArrayOfPolylines(2)
            line.AddVertex(*transform(x-0.5, -0.5,  z-0.5, trsf))
            line.AddVertex(*transform(x-0.5, sy-0.5, z-0.5, trsf))
            group.AddPrimitiveArray(line)    
            
#&
def iterate4(mv, mv2, group, trsf, min_val, val):
##def iterate4(int [:,:,::1] mv, long [:,:,::1] mv2, group, trsf, int min_val, int val):

##  cdef int x, y, z, N1, N2, N3
##  cdef bint t,b,l,r,f,back
    
    sx = mv.shape[0]
    sy = mv.shape[1]
    sz = mv.shape[2]
     
    min_val -= 1 
        
    for x in range(sx):
        for y in range (sy):
            for z in range (sz):
                t = 0
                b = 0
                r = 0
                l = 0
                f = 0
                back = 0
        
                if min_val < mv[x,y,z] <= val and mv[x,y,z] != 0 :
                    
                    if mv2[x,y,z] == 1:
                        continue
                    
                    if x == 0:
                        l = True
                        
                    elif mv2[x-1,y,z] == 1:
                        l = True                  
                        
                    elif mv[x-1,y,z] > val or mv[x-1,y,z] <= min_val:
                        l = True
                
                        
                    if y == 0:
                        f = True
                    
                    elif mv2[x,y-1,z] == 1:
                        f = True                            
                        
                    elif  mv[x,y-1,z] > val or mv[x,y-1,z] <= min_val:
                        f = True
        
                    if z == 0:
                        b = True
                    
                    elif mv2[x,y,z-1] == 1:
                        b = True                            
                        
                    elif  mv[x,y,z-1] > val or mv[x,y,z-1] <= min_val:
                        b = True
    
                    if x==sx-1:
                        r = True 
                    
                    elif mv2[x+1,y,z] == 1:
                        r = True                           
                        
                    elif  mv[x+1,y,z] > val or mv[x+1,y,z] <= min_val:
                        r = True
                        
                    if y==sy-1:
                        back = True
                        
                    elif mv2[x,y+1,z] == 1:
                        back = True                      
                        
                    elif  mv[x,y+1,z] > val or mv[x,y+1,z] <= min_val:
                        back = True
        
                    if z==sz-1:
                        t = True
                    
                    elif mv2[x,y,z+1] == 1:
                        t = True
                        
                    elif  mv[x,y,z+1] > val or mv[x,y,z+1] <= min_val:
                        t = True
                                       
                    display_v(group, trsf, x,y,z, t,b,l,r,f,back, mv[x,y,z])
        
    return True
